<?php
declare(strict_types=1);

namespace Inti\Wp\Plugin\Service;

use Inti\Wp\Plugin\Exception;
use Inti\Wp\Plugin\PluginInterface;
use Throwable;
use function is_array;
use function is_string;

/**
 * Runs all installed plugins.
 *
 * @package Inti\Wp\Plugin\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-plugin
 */
final class PluginRunner implements PluginRunnerInterface
{
    private PluginManager $manager;

    public function __construct(PluginManager $manager)
    {
        $this->manager = $manager;
    }

    public function run(array $plugins): void
    {
        foreach ($plugins as $key => $plugin) {
            if (is_string($key) && is_array($plugin)) {
                $options = $plugin;
                $plugin = $key;
            }

            $plugin = $this->getPlugin($plugin);
            $plugin->run($options ?? null);
        }
    }

    private function getPlugin($plugin): PluginInterface
    {
        if (is_string($plugin)) {
            try {
                $plugin = $this->manager->get($plugin);
            } catch (Throwable $e) {
                throw new Exception\PluginCouldNotBeCreatedException($plugin, 0, $e);
            }
        }

        if (!$plugin instanceof PluginInterface) {
            throw new Exception\InvalidPluginException($plugin);
        }

        return $plugin;
    }
}
