<?php
declare(strict_types=1);

namespace IntiTest\Wp\Plugin\Service;

use Inti\Wp\Plugin\Exception\InvalidPluginException;
use Inti\Wp\Plugin\Exception\PluginCouldNotBeCreatedException;
use Inti\Wp\Plugin\Service\PluginManager;
use Inti\Wp\Plugin\Service\PluginRunner;
use Inti\Wp\Plugin\Service\PluginRunnerInterface;
use IntiTest\Wp\Plugin\Mock\BrokenPlugin;
use IntiTest\Wp\Plugin\Mock\MockPlugin;
use IntiTest\Wp\Plugin\TestCaseTrait;
use PHPUnit\Framework\TestCase;
use stdClass;

/**
 * Class PluginRunnerTest
 *
 * @package IntiTest\Wp\Plugin\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-plugin
 */
final class PluginRunnerTest extends TestCase
{
    use TestCaseTrait;

    public function testTheRunnerIsRegistered()
    {
        $this->assertContainerHasService(PluginRunner::class);
    }

    public function testTheRunnerIsRegisteredAsDefault()
    {
        $this->assertServiceInstanceOf(PluginRunner::class, PluginRunnerInterface::class);
    }

    public function testWillThrowExceptionForCreationErrors()
    {
        $runner = $this->createRunner();

        $this->expectException(PluginCouldNotBeCreatedException::class);
        $runner->run([BrokenPlugin::class]);
    }

    public function testWillThrowExceptionForInvalidPlugin()
    {
        $runner = $this->createRunner();

        $this->expectException(InvalidPluginException::class);
        $runner->run([new stdClass()]);
    }

    public function testWillPassOptionsToPlugins()
    {
        $expected = [
            'option1' => 1,
            'option2' => 2,
        ];

        $runner = $this->createRunner();
        $runner->run([
            MockPlugin::class => $expected,
        ]);

        $manager = $this->container->get(PluginManager::class);
        /** @var MockPlugin $plugin */
        $plugin = $manager->get(MockPlugin::class);

        $this->assertEquals($expected, $plugin->getOptions());
    }
}
